﻿using System;

namespace WCS.Entity.Protocol.RGV
{
    /// <summary>
    /// RGV运行状态
    /// </summary>
    public enum RGVRunStatus : short
    {
        空闲 = 1,
        取货移动 = 2,
        取货 = 3,
        放货移动 = 4,
        放货 = 5
    }

    /// <summary>
    /// RGV运行模式
    /// </summary>
    public enum RGVMode : short
    {
        自动 = 1,
        半自动 = 2,
        手动 = 3,
        维护 = 4
    }

    /// <summary>
    /// RGV任务类型
    /// </summary>
    public enum RGVTaskType : short
    {
        取货 = 1,
        放货 = 2,
        移动 = 3,
        码盘 = 4,
        拆盘 = 5,
        变更放货站台 = 6,

        /// <summary>
        /// 环穿、双工位使用
        /// </summary>
        变更取货站台 = 7,
    }

    /// <summary>
    /// RGV故障
    /// </summary>
    [Flags]
    public enum RGVFault : ushort
    {
        RGV前后防撞条故障 = 1 << 0,
        RGV载货极限光电故障 = 1 << 1,
        RGV输送线变频器故障 = 1 << 2,
        RGV行走变频器故障 = 1 << 3,
        RGV扫码器故障 = 1 << 4,
        RGV位置值故障 = 1 << 5,
        RGV定位失败 = 1 << 6,
        RGV取放货时位置偏差过大 = 1 << 7,
        RGV急停按下 = 1 << 8,
        主柜急停按下 = 1 << 9,
        站台货物超出 = 1 << 10,
        RGV输送线超时 = 1 << 11,
        RGV输送线光电异常 = 1 << 12,
        RGV任务号与站台任务号不匹配 = 1 << 13,
        RGV不支持此任务类型 = 1 << 14,
        RGV目标站台与交互站台号不一致 = 1 << 15,
    }

    /// <summary>
    /// RGV工位故障
    /// </summary>
    [Flags]
    public enum RGVPlatformFault : ushort
    {
        RGV站台号错误 = 1 << 0,
        RGV任务类型错误 = 1 << 1,
        RGV任务号错误 = 1 << 2,
        RGV货物超出故障 = 1 << 3,
        RGV输送线变频器故障2 = 1 << 4,
        RGV输送线传感器异常 = 1 << 5,
        RGV输送线运行超时 = 1 << 6,
        RGV输送线总故障 = 1 << 7,
        RGV硬极限故障 = 1 << 8,
        RGV软极限故障 = 1 << 9,
        RGV货物超出故障2 = 1 << 10,
        RGV行走测距信号丢失 = 1 << 11,
        RGV行走变频器故障 = 1 << 12,
        RGV = 1 << 13,
        RGV2 = 1 << 14,
        RGV急停按下 = 1 << 15,
    }

    /// <summary>
    /// RGV状态
    /// </summary>
    [Flags]
    public enum RGVStatus : ushort
    {
        RGV与站台交互完成 = 1 << 0,
        RGV前方有障暂停运行 = 1 << 1,
        RGV = 1 << 2,
        RGV2 = 1 << 3,
        RGV3 = 1 << 4,
        RGV4 = 1 << 5,
        RGV5 = 1 << 6,
        RGV6 = 1 << 7,
        RGV运行 = 1 << 8,
        RGV移动中 = 1 << 9,
        RGV取货中 = 1 << 10,
        RGV放货中 = 1 << 11,
        RGV变址运行 = 1 << 12,
        RGV定位完成 = 1 << 13,
        RGV有货光电 = 1 << 14,
        RGV与站台交互中 = 1 << 15,
    }

    /// <summary>
    /// RGV工位状态
    /// </summary>
    [Flags]
    public enum RGVPlatformStatus : ushort
    {
        RGV任务挂起 = 1 << 0,
        RGV移动中 = 1 << 1,
        RGV取货中 = 1 << 2,
        RGV放货中 = 1 << 3,
        RGV定位完全 = 1 << 4,
        RGV有货光电 = 1 << 5,
        RGV与站台交互中 = 1 << 6,
        RGV与站台交互完 = 1 << 7,
        RGV = 1 << 8,
        RGV2 = 1 << 9,
        RGV3 = 1 << 10,
        RGV4 = 1 << 11,
        RGV5 = 1 << 12,
        RGV6 = 1 << 13,
        RGV7 = 1 << 14,
        RGV8 = 1 << 15,
    }
}