﻿using SqlSugar;
using System;
using WCS.Entity.Protocol;

namespace WCS.Entity
{
    /// <summary>
    /// 任务表
    /// </summary>
    [SugarTable(nameof(WCS_TASK), "任务表")]
    public class WCS_TASK : OBJ
    {
        /// <summary>
        /// 生产编码
        /// </summary>
        [SugarColumn(ColumnDescription = "生产编码", Length = 50)]
        public string BARCODE { get; set; }

        /// <summary>
        /// 任务类型
        /// </summary>
        [SugarColumn(ColumnDescription = "任务类型")]
        public TaskType TYPE { get; set; }

        /// <summary>
        /// 任务状态
        /// </summary>
        [SugarColumn(ColumnDescription = "任务状态")]
        public TaskStatus STATUS { get; set; }

        /// <summary>
        /// 起始地址
        /// </summary>
        [SugarColumn(ColumnDescription = "起始地址", Length = 20)]
        public string ADDRFROM { get; set; }

        /// <summary>
        /// 目标地址
        /// </summary>
        [SugarColumn(ColumnDescription = "目标地址", Length = 20)]
        public string ADDRTO { get; set; }

        /// <summary>
        /// 下一个地址
        /// </summary>
        [SugarColumn(ColumnDescription = "下一个地址", Length = 20)]
        public string ADDRNEXT { get; set; }

        /// <summary>
        /// 巷道
        /// </summary>
        [SugarColumn(ColumnDescription = "巷道", Length = 10, IsNullable = true)]
        public string TUNNEL { get; set; }

        /// <summary>
        /// 楼层
        /// </summary>
        [SugarColumn(ColumnDescription = "楼层")]
        public int FLOOR { get; set; }

        /// <summary>
        /// 设备
        /// </summary>
        [SugarColumn(ColumnDescription = "设备", Length = 20, IsNullable = true)]
        public string DEVICE { get; set; }

        /// <summary>
        /// 堆垛机放货站台
        /// </summary>
        [SugarColumn(ColumnDescription = "堆垛机放货站台", Length = 20, IsNullable = true)]
        public string SRMSTATION { get; set; }

        /// <summary>
        ///创建时间
        /// </summary>
        [SugarColumn(ColumnDescription = "创建时间")]
        public DateTime CREATETIME { get; set; } = DateTime.Now;

        /// <summary>
        /// 开始时间
        /// </summary>
        [SugarColumn(ColumnDescription = "开始时间")]
        public DateTime? STARTTIME { get; set; }

        /// <summary>
        /// 结束时间
        /// </summary>
        [SugarColumn(ColumnDescription = "结束时间", IsNullable = true)]
        public DateTime? ENDTIME { get; set; }

        /// <summary>
        /// 单号
        /// </summary>
        [SugarColumn(ColumnDescription = "单号", IsNullable = true)]
        public int DOCID { get; set; }

        /// <summary>
        /// 托盘类型
        /// </summary>
        [SugarColumn(ColumnDescription = "托盘类型", IsNullable = true)]
        public short PALLETTYPE { get; set; }

        /// <summary>
        /// 产线
        /// </summary>
        [SugarColumn(ColumnDescription = "产线", IsNullable = true)]
        public short PRODLINE { get; set; }

        /// <summary>
        /// 满盘数量
        /// </summary>
        [SugarColumn(ColumnDescription = "满盘数量", IsNullable = true)]
        public short FULLQTY { get; set; }

        /// <summary>
        /// 高度
        /// </summary>
        [SugarColumn(ColumnDescription = "高度", IsNullable = true)]
        public short HEIGHT { get; set; }

        /// <summary>
        /// WMS任务号
        /// </summary>
        [SugarColumn(ColumnDescription = "WMS任务号")]
        public int WMSTASK { get; set; }

        /// <summary>
        /// 任务组ID
        /// </summary>
        [SugarColumn(ColumnDescription = "任务组ID", IsNullable = true)]
        public string TaskGroupKey { get; set; }

        /// <summary>
        /// 更新状态
        /// </summary>
        [SugarColumn(ColumnDescription = "更新状态")]
        public TaskStatus UPLOADED { get; set; }

        /// <summary>
        /// AGV任务号
        /// </summary>
        [SugarColumn(ColumnDescription = "AGV任务号", IsNullable = true)]
        public int AGVTASKID { get; set; }

        /// <summary>
        /// AGV任务详细信息
        /// </summary>
        [Navigate(NavigateType.OneToOne, nameof(AGVTASKID))]
        public WCS_AGVTask AGVTASK { get; set; }

        /// <summary>
        /// 优先级
        /// </summary>
        [SugarColumn(ColumnDescription = "优先级")]
        public int Priority { get; set; }

        /// <summary>
        /// 手动处理备注
        /// </summary>
        [SugarColumn(ColumnDescription = "手动处理备注", IsNullable = true)]
        public string ManualRemarks { get; set; }

        /// <summary>
        /// 产品长度
        /// </summary>
        [SugarColumn(ColumnDescription = "产品长度", IsNullable = true)]
        public decimal Length { get; set; }

        /// <summary>
        /// 产品编码
        /// </summary>
        [SugarColumn(ColumnDescription = "产品编码", IsNullable = true)]
        public string MaterialCode { get; set; }
    }

    /// <summary>
    /// 任务表
    /// </summary>
    [SugarTable(nameof(WCS_TASK_OLD), "任务表")]
    public class WCS_TASK_OLD : OBJ
    {
        /// <summary>
        /// 生产编码
        /// </summary>
        [SugarColumn(ColumnDescription = "生产编码", Length = 50)]
        public string BARCODE { get; set; }

        /// <summary>
        /// 任务类型
        /// </summary>
        [SugarColumn(ColumnDescription = "任务类型")]
        public TaskType TYPE { get; set; }

        /// <summary>
        /// 任务状态
        /// </summary>
        [SugarColumn(ColumnDescription = "任务状态")]
        public TaskStatus STATUS { get; set; }

        /// <summary>
        /// 起始地址
        /// </summary>
        [SugarColumn(ColumnDescription = "起始地址", Length = 20)]
        public string ADDRFROM { get; set; }

        /// <summary>
        /// 目标地址
        /// </summary>
        [SugarColumn(ColumnDescription = "目标地址", Length = 20)]
        public string ADDRTO { get; set; }

        /// <summary>
        /// 下一个地址
        /// </summary>
        [SugarColumn(ColumnDescription = "下一个地址", Length = 20)]
        public string ADDRNEXT { get; set; }

        /// <summary>
        /// 巷道
        /// </summary>
        [SugarColumn(ColumnDescription = "巷道", Length = 10)]
        public string TUNNEL { get; set; }

        /// <summary>
        /// 楼层
        /// </summary>
        [SugarColumn(ColumnDescription = "楼层")]
        public int FLOOR { get; set; }

        /// <summary>
        /// 设备
        /// </summary>
        [SugarColumn(ColumnDescription = "设备", Length = 20)]
        public string DEVICE { get; set; }

        /// <summary>
        /// 堆垛机放货站台
        /// </summary>
        [SugarColumn(ColumnDescription = "堆垛机放货站台", Length = 20)]
        public string SRMSTATION { get; set; }

        /// <summary>
        ///创建时间
        /// </summary>
        [SugarColumn(ColumnDescription = "创建时间")]
        public DateTime CREATETIME { get; set; } = DateTime.Now;

        /// <summary>
        /// 开始时间
        /// </summary>
        [SugarColumn(ColumnDescription = "开始时间")]
        public DateTime? STARTTIME { get; set; }

        /// <summary>
        /// 结束时间
        /// </summary>
        [SugarColumn(ColumnDescription = "结束时间")]
        public DateTime? ENDTIME { get; set; }

        /// <summary>
        /// 单号
        /// </summary>
        [SugarColumn(ColumnDescription = "单号")]
        public int DOCID { get; set; }

        /// <summary>
        /// 托盘类型
        /// </summary>
        [SugarColumn(ColumnDescription = "托盘类型")]
        public short PALLETTYPE { get; set; }

        /// <summary>
        /// 产线
        /// </summary>
        [SugarColumn(ColumnDescription = "产线")]
        public short PRODLINE { get; set; }

        /// <summary>
        /// 满盘数量
        /// </summary>
        [SugarColumn(ColumnDescription = "满盘数量")]
        public short FULLQTY { get; set; }

        /// <summary>
        /// 高度
        /// </summary>
        [SugarColumn(ColumnDescription = "高度")]
        public short HEIGHT { get; set; }

        /// <summary>
        /// WMS任务号
        /// </summary>
        [SugarColumn(ColumnDescription = "WMS任务号")]
        public int WMSTASK { get; set; }

        /// <summary>
        /// 任务组ID
        /// </summary>
        [SugarColumn(ColumnDescription = "任务组ID")]
        public string TaskGroupKey { get; set; }

        /// <summary>
        /// 更新状态
        /// </summary>
        [SugarColumn(ColumnDescription = "更新状态")]
        public TaskStatus UPLOADED { get; set; }

        /// <summary>
        /// AGV任务号
        /// </summary>
        [SugarColumn(ColumnDescription = "AGV任务号")]
        public int AGVTASKID { get; set; }

        /// <summary>
        /// AGV任务详细信息
        /// </summary>
        [Navigate(NavigateType.OneToOne, nameof(AGVTASKID))]
        public WCS_AGVTask AGVTASK { get; set; }

        /// <summary>
        /// 优先级
        /// </summary>
        [SugarColumn(ColumnDescription = "优先级")]
        public int Priority { get; set; }

        /// <summary>
        /// 手动处理备注
        /// </summary>
        [SugarColumn(ColumnDescription = "手动处理备注")]
        public string ManualRemarks { get; set; }

        /// <summary>
        /// 产品长度
        /// </summary>
        [SugarColumn(ColumnDescription = "产品长度")]
        public decimal Length { get; set; }

        /// <summary>
        /// 产品编码
        /// </summary>
        [SugarColumn(ColumnDescription = "产品编码")]
        public string MaterialCode { get; set; }
    }

    public enum TaskType
    {
        组盘 = 0,
        入库 = 1,
        出库 = 2,
        移库 = 3,
        输送 = 5,
    }

    public enum TaskStatus
    {
        新建 = 0,
        执行中 = 2,
        入库交管结束 = 3,
        堆垛机执行 = 5,
        堆垛机完成 = 6,
        出库RGV执行 = 9,
        已完成 = 99,
        已取消 = 106
    }
}
