using DbHelper;
using Log;
using Newtonsoft.Json;
using SqlSugar;
using System.Reflection;
using System.Text;
using WCS.Core;
using WCS.Core.DataTrans;
using WCS.Entity;
using WCS.Entity.Protocol;
using WCS.Entity.Protocol.BCR;
using WCS.Entity.Protocol.Station;
using WCS.Virtual_PLC;
using WCS.WebApi.WMS;

namespace WCS.Service
{
    public class Worker : BackgroundService
    {
        private readonly ILogger<Worker> _logger;

        public Worker(ILogger<Worker> logger)
        {
            _logger = logger;
        }

        protected override async System.Threading.Tasks.Task ExecuteAsync(CancellationToken stoppingToken)
        {
            if (stoppingToken.IsCancellationRequested)
                return;

            #region ־

            var logConfig = JsonConvert.DeserializeObject<LogConfig>(await File.ReadAllTextAsync("config.json", Encoding.Default, stoppingToken));
            LogHelper.SetConfigInfo(logConfig!);

            #endregion ־

            InfoLog.INFO_INIT("1111");

            _logger.LogInformation("WCSʼ");
            //InfoLog.INFO_INIT("WCSʼ");
            Configs.DebugRedisUrl = "127.0.0.1";
            Configs.ProtocolProxyBaseType = typeof(ProtocolProxy);
            Encoding.RegisterProvider(CodePagesEncodingProvider.Instance);
            Configs.StringEncoding = Encoding.UTF8;

            Db.CreateContext(new ConnectionConfig()
            {
                ConnectionString = AppSettings.Config.GetConnectionString("WCSDB"),
                DbType = DbType.SqlServer
            }, "WCSDB");
            Db.SetDefaultDbContextType("WCSDB");
            Db.Do(db =>
            {
                //TODO:DbMaintenance.CreateDatabase()ûãûжӦݿĻȻҪֶ½һ
                db.Default.DbMaintenance.CreateDatabase();
                db.Default.CodeFirst.InitTables(typeof(WCS_CMD));
                db.Default.CodeFirst.InitTables(typeof(WCS_PLC));
                db.Default.CodeFirst.InitTables(typeof(WCS_DATABLOCK));
                db.Default.CodeFirst.InitTables(typeof(WCS_DEVICE));
                db.Default.CodeFirst.InitTables(typeof(WCS_PATH));
                db.Default.CodeFirst.InitTables(typeof(WCS_PATHPOINT));
                db.Default.CodeFirst.InitTables(typeof(WCS_ROUTE));
                db.Default.CodeFirst.InitTables(typeof(WCS_TASK));
                db.Default.CodeFirst.InitTables(typeof(WCS_TASK_OLD));
                db.Default.CodeFirst.InitTables(typeof(WCS_EXCEPTION));
                db.Default.CodeFirst.InitTables(typeof(WCS_SystemConfig));
                db.Default.CodeFirst.InitTables(typeof(WCS_AGVTask));
                db.Default.CodeFirst.InitTables(typeof(WCS_DEVICEPROTOCOL));
                db.Default.CodeFirst.InitTables(typeof(WCS_GROUPMEMBER));
                db.Default.CodeFirst.InitTables(typeof(WCS_MAPPINGENTRY));
                db.Default.CodeFirst.InitTables(typeof(WCS_USERS));
                db.Default.CodeFirst.InitTables(typeof(WCS_StatusLog));
                db.Default.CodeFirst.InitTables(typeof(WCS_BCR80));
                db.Default.CodeFirst.InitTables(typeof(WCS_RGV520));
                db.Default.CodeFirst.InitTables(typeof(WCS_RGV521));
                db.Default.CodeFirst.InitTables(typeof(WCS_RGV523));
                db.Default.CodeFirst.InitTables(typeof(WCS_SRM520));
                db.Default.CodeFirst.InitTables(typeof(WCS_SRM521));
                db.Default.CodeFirst.InitTables(typeof(WCS_SRM537));
                db.Default.CodeFirst.InitTables(typeof(WCS_Station520));
                db.Default.CodeFirst.InitTables(typeof(WCS_Station521));
                db.Default.CodeFirst.InitTables(typeof(WCS_Station523));
            });

            //нṹҪĽṹ
            List<PLCData> list = new List<PLCData>();
            Db.Do(db =>
            {
                var dataBlocks = db.Default.Queryable<WCS_DATABLOCK>().Includes(v => v.PLC).ToList();
                foreach (var dataBlock in dataBlocks)
                {
                    list.Add(new PLCData()
                    {
                        IP = dataBlock.PLC.IP,
                        DB = dataBlock.NO,
                        Length = dataBlock.LENGTH,
                        DataLength = dataBlock.DATALENGTH,
                    });
                }
            });

            PlcData.Init("127.0.0.1,database=0").InitPlcData(list);
            //־¼s
            Configs.PublishEvent += () =>
            {
                WMS.UploadDevInfo();
                ProtocolProxy.Do();
            };
            //쳣
            Configs.UploadException = (d, s) =>
            {
                if (s == "ӿڵ") return;
                if (ProtocolProxy.AllDatas.ContainsKey(d))
                {
                    ProtocolProxy.AllDatas[d].Info = s;
                    ProtocolProxy.AllDatas[d].Frame = LogicHandler.Frame;
                }
                WMS.TaskException(d, s);
            };

            //LogicHandler.DbLog = Helpers.LogHelper.AddWCS_EXCEPTION;

            PlcAccessor.Creater = new PLCAccessors.PlcAccessorsCreater();
            try
            {
                Db.Do(db =>
                {
                    var items = db.Default.Queryable<WCS_DEVICEPROTOCOL>()
                    .Includes(d => d.DEVICE, r => r.ROUTES)
                    .Includes(d => d.DEVICE, p => p.PATHS)
                    .Includes(d => d.DB, p => p.PLC).ToArray();
                    items.Select(v => v.Data()).ToArray();
                    LogicHandler.AllObjects.AddRange(items);

                    var devices = db.Default.Queryable<WCS_DEVICE>()
                                            .Includes(v => v.ROUTES)
                                            .Includes(v => v.PATHS)
                                            .Includes(v => v.DEVICEGROUP, g => g.GROUP)
                                            .AsNavQueryable()
                                            .Includes(v => v.DEVICEGROUP, g => g.MEMBER, d => d.DEVICEPROTOCOLS, p => p.DB, c => c.PLC)
                                            .Includes(v => v.DEVICEGROUP, g => g.MEMBER, d => d.DEVICEPROTOCOLS, p => p.DEVICE)
                                            .Includes(v => v.DEVICEPROTOCOLS, p => p.DB, c => c.PLC)
                                            .Includes(v => v.DEVICEPROTOCOLS, p => p.DEVICE)
                                            .ToList();
                    LogicHandler.AllObjects.AddRange(devices);
                });

                #region 豸չ

                //WCS_DEVICEExtension.AddFlag(DF.һ¥RGVŻ, "G1035", "G1044", "G1053", "G1062");
                //WCS_DEVICEExtension.AddFlag(DF.SRM, "SRM1", "SRM2", "SRM3", "SRM4", "SRM5", "SRM6", "SRM7", "SRM8");
                //WCS_DEVICEExtension.AddFlag(DF.̨, "G1469", "G1561", "G1538", "G1574", "G1509");
                //WCS_DEVICEExtension.AddFlag(DF.SRMƷȡ, "1040", "1041", "1042", "1043", "1049", "1050", "1051", "1052");
                //WCS_DEVICEExtension.AddFlag(DF.SRMƷȡ, "1058", "1059", "1060", "1061", "1067", "1068");
                //WCS_DEVICEExtension.AddFlag(DF.SRMBOPPȡ, "1195", "1194", "1193", "1192", "1204", "1203", "1202", "1201");
                //WCS_DEVICEExtension.AddFlag(DF.SRMBOPPȡ, "1213", "1212", "1210", "1211", "1220", "1219", "1230", "1228");
                //WCS_DEVICEExtension.AddFlag(DF.SRM̨Ż, "1473", "1476", "1474", "1475", "1491", "1492", "1493", "1494");
                //WCS_DEVICEExtension.AddFlag(DF.SRM̨Ż, "1520", "1521", "1522", "1523", "1545", "1546", "1451", "1453");
                //WCS_DEVICEExtension.AddFlag(DF.SRMͿȡ, "1431", "1432", "1422", "1423", "1424", "1425", "1415", "1416");
                //WCS_DEVICEExtension.AddFlag(DF.SRMͿȡ, "1605", "1606", "1406", "1407", "1408", "1409");
                //WCS_DEVICEExtension.AddFlag(DF.SRMͿŻ, "1283", "1284", "1290", "1291", "1292", "1293", "1299", "1300");
                //WCS_DEVICEExtension.AddFlag(DF.SRMͿŻ, "1301", "1302", "1308", "1309", "1310", "1311");
                //WCS_DEVICEExtension.AddFlag(DF.ͿRGV, "RGV9", "RGV10", "RGV11", "RGV12", "RGV13", "RGV14");
                //WCS_DEVICEExtension.AddFlag(DF.ͿRGVȡ豸, "G2", "G3", "G5", "G7", "G9", "G11");
                //WCS_DEVICEExtension.AddFlag(DF.ͿRGVŻ豸, "G1", "G4", "G6", "G8", "G10");
                //WCS_DEVICEExtension.AddFlag(DF.ͿRGVȡվ̨, "1285", "1286", "1294", "1295", "1303", "1304", "1312", "1313");
                //WCS_DEVICEExtension.AddFlag(DF.ͿRGVȡվ̨, "1391", "1392", "1399", "1400");
                //WCS_DEVICEExtension.AddFlag(DF.ͿRGVȡվ̨, "1285", "1286", "1294", "1295", "1303", "1304", "1312", "1313");
                //WCS_DEVICEExtension.AddFlag(DF.ͿRGVȡվ̨, "1391", "1392", "1399", "1400");
                //WCS_DEVICEExtension.AddFlag(DF.BOPPRGV, "RGV1", "RGV2", "RGV3", "RGV4", "RGV5", "RGV6", "RGV7");
                //WCS_DEVICEExtension.AddFlag(DF.BOPPRGVȡ豸, "G19", "G23");
                //WCS_DEVICEExtension.AddFlag(DF.BOPPRGVŻ豸, "G12", "G13", "G14", "G15", "G16");

                #endregion 豸չ

                #region е߼

                var managerTypes = Assembly.GetExecutingAssembly().GetTypes().Where(v => v.IsSubclassOf(typeof(LogicHandler)) && !v.IsGenericType && !v.IsAbstract).ToArray();
                foreach (var type in managerTypes)
                {
                    var m = Activator.CreateInstance(type);
                    LogicHandler.AddManager(m as LogicHandler);
                }

                LogicHandler.StartAll();

                #endregion е߼

                _logger.LogInformation("WCSɹ");
                //InfoLog.INFO_INIT("WCSɹ");
            }
            catch (Exception ex)
            {
                _logger.LogError("WCSʧ{0}", ex.Message);
                //InfoLog.INFO_INIT($"WCSʧ{ex.Message}");
            }
        }

        public override System.Threading.Tasks.Task StopAsync(CancellationToken cancellationToken)
        {
            _logger.LogError("WCSر");
            //InfoLog.INFO_INIT("WCSر");
            LogicHandler.StopAll();
            return base.StopAsync(cancellationToken);
        }
    }
}
